/* foXcontrol
 * Copyright (C) 2000-2001 Josua Groeger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "FDAppMainWindow.h"
#include "FDCRoot.h"
#include "FDCSection.h"

FXIMPLEMENT (FDCompoTreeItem, FXTreeItem, NULL, 0)

FXDEFMAP (FDAppMainWindow) FDAppMainWindowMap[]=
{
  FXMAPFUNC (SEL_CLICKED, FDAppMainWindow::ID_TREELIST,           FDAppMainWindow::onTreeList),
  FXMAPFUNC (SEL_COMMAND, FDAppMainWindow::ID_ROOTCOMPONENTAPPLY, FDAppMainWindow::onRootComponentApply)
};

FXIMPLEMENT (FDAppMainWindow, FXMainWindow, FDAppMainWindowMap, ARRAYNUMBER (FDAppMainWindowMap))

FDAppMainWindow::FDAppMainWindow (FDApp *a, const FXString &name, FXIcon *ic= NULL, FXIcon *mi= NULL,
                                  FXuint opts= DECOR_ALL, FXint x= 0, FXint y= 0, FXint w= 0, FXint h= 0):
                 FXMainWindow (a, name, ic, mi, opts, x,y,w,h, 0,0)
{
  // Some widgets
  statusbar= new FXStatusbar (this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER);
  FXSplitter *splitter= new FXSplitter  (this, LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_TRACKING);
  
  FXPacker *treeFrame= new FXPacker (splitter, FRAME_NORMAL, 0,0,225,0, 0,0,0,0,0,0);
  treeList=new FXTreeList (treeFrame, 0, this, ID_TREELIST, LAYOUT_FILL_X|LAYOUT_FILL_Y| \
                           TREELIST_BROWSESELECT|TREELIST_SHOWS_LINES| \
                           TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES);
  
  FXScrollWindow *scrollWindow= new FXScrollWindow (splitter, LAYOUT_FILL_X|LAYOUT_FILL_Y);
  FXPacker *scrollFrame= new FXPacker (scrollWindow, FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,
                                       0,0,0,0, 0,0,0,0,0,0);

  // Root component (the one appearing when you start foXcontrol)
  FDComponent *rootComponent= new FDCRoot (scrollFrame, LAYOUT_FILL_X|LAYOUT_FILL_Y,
                                           this, MKUINT (ID_ROOTCOMPONENTAPPLY, SEL_COMMAND), NULL,
                                           getApp (), MKUINT (FXApp::ID_QUIT, SEL_COMMAND), NULL,
                                           0,0,0,0, 0,0,0,0);
 
  FDCompoTreeItem *rootItem= new FDCompoTreeItem (rootComponent, a);
  treeList->addItemLast (NULL, rootItem);

  // Insert all installed foXcontrol components
  constructDict= new FDComponentConstructDict (a);
  constructDict->insert ("foXcontrol");
  
  FDComponent *tmpComponent;
  
  short pos= constructDict->last ();
  while (pos>= 0)
  {
    tmpComponent= constructDict->data (pos) (scrollFrame, LAYOUT_FILL_X|LAYOUT_FILL_Y,
                                             NULL, 0, NULL, this, MKUINT (ID_TREELIST, SEL_CLICKED), rootItem,
                                             0,0,0,0, 0,0,0,0);
    tmpComponent->hide ();

    // Parse the tree list for the proper section item
    FDCompoTreeItem *sectionItem= (FDCompoTreeItem*) treeList->getFirstItem ();
    if (sectionItem) do
    {
      if (!strcmp (tmpComponent->reg ().readStringEntry ("SETTINGS", "section", ""),
                   ((FDComponent*) treeList->getItemData (sectionItem))->reg ().readStringEntry
                                                                 ("SETTINGS", "caption", ""))) break;
     
      sectionItem= (FDCompoTreeItem*) sectionItem->getNext ();
    }
    while (sectionItem);

    // Insert a new section item if necessary
    if (!sectionItem)
    {
      FDCSection *section= new FDCSection (scrollFrame, LAYOUT_FILL_X|LAYOUT_FILL_Y,
                                           NULL, 0, NULL, this, MKUINT (ID_TREELIST, SEL_CLICKED), rootItem,
                                           0,0,0,0, 0,0,0,0);
      section->hide ();
      section->setData ((void*) tmpComponent->reg ().readStringEntry ("SETTINGS", "section", ""));

      sectionItem= new FDCompoTreeItem (section, a);
      treeList->addItemLast (NULL, sectionItem);
      treeList->expandTree (sectionItem);
    }

    // Insert a new item/component
    FDCompoTreeItem *item= new FDCompoTreeItem (tmpComponent, a);
    treeList->addItemLast (sectionItem, item);
              
    pos= constructDict->prev (pos);
  }

  // Initialization
  currentComponent= NULL;
  handle (this, MKUINT (ID_ROOTCOMPONENTAPPLY, SEL_COMMAND), NULL);
  handle (this, MKUINT (ID_TREELIST, SEL_CLICKED), rootItem);
}

FDAppMainWindow::~FDAppMainWindow ()
{
  delete constructDict;
}

long FDAppMainWindow::onTreeList (FXObject*, FXSelector, void *ptr)
{
  // Hide current component
  if (currentComponent) currentComponent->handle (this, MKUINT (FXWindow::ID_HIDE, SEL_COMMAND), NULL);

  // Get new current item/component
  FDCompoTreeItem *currentItem= (FDCompoTreeItem*) ptr;
  treeList->setCurrentItem (currentItem); 
  currentComponent= (FDComponent*) treeList->getItemData (currentItem);

  // Show new current component
  currentComponent->handle (this, MKUINT (FXWindow::ID_SHOW, SEL_COMMAND), NULL);
  setTitle (FXString ("foXcontrol: ")+ currentComponent->reg ().readStringEntry ("SETTINGS", "caption", ""));
  statusbar->getStatusline ()->setNormalText
                                 (currentComponent->reg ().readStringEntry ("SETTINGS", "description", ""));

  return 1;
}

long FDAppMainWindow::onRootComponentApply (FXObject*, FXSelector, void*)
{
  if (getApp ()->reg ().readUnsignedEntry ("SETTINGS", "statusbar", true))
       statusbar->handle (NULL, MKUINT (FXWindow::ID_SHOW, SEL_COMMAND), NULL);
  else statusbar->handle (NULL, MKUINT (FXWindow::ID_HIDE, SEL_COMMAND), NULL);

  return 1;
}
