/* foXcontrol
 * Copyright (C) 2000-2001 Josua Groeger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef FDAPPMAINWINDOW_H
#define FDAPPMAINWINDOW_H

#include <foXdesktop/foXdesktop.h>

// Each tree list item holds a component as its data.
// Subclassing FXTreeItem is necessary for properly deleting it.
class FDCompoTreeItem: public FXTreeItem
{
  FXDECLARE (FDCompoTreeItem)
  public:
    FDCompoTreeItem (FDComponent *compo, FDApp *a): FXTreeItem
                    (compo->reg ().readStringEntry ("SETTINGS", "caption", ""),
                     a->ico().insert (compo->reg ().readStringEntry ("SETTINGS", "miniicon", "")),
                     a->ico().insert (compo->reg ().readStringEntry ("SETTINGS", "miniicon", "")),
                     compo) {};

    virtual ~FDCompoTreeItem ()
    {
      delete (FDComponent*) data;
    };
    
  protected:
    FDCompoTreeItem (): FXTreeItem () {};
};

class FDAppMainWindow: public FXMainWindow
{
  FXDECLARE (FDAppMainWindow)
  private:
    FXStatusbar *statusbar;
    FXTreeList *treeList;
    FDComponentConstructDict *constructDict;
    FDComponent *currentComponent;

  protected:
    FDAppMainWindow () {};

  public:
    FDAppMainWindow (FDApp *a, const FXString &name, FXIcon *ic= NULL, FXIcon *mi= NULL,
                     FXuint opts= DECOR_ALL, FXint x=0, FXint y=0, FXint w=0, FXint h=0);
    virtual ~FDAppMainWindow ();
    
    enum
    {
      ID_TREELIST= FXMainWindow::ID_LAST,
      ID_ROOTCOMPONENTAPPLY,
      ID_LAST
    };
    
    long onTreeList           (FXObject*, FXSelector, void*);
    long onRootComponentApply (FXObject*, FXSelector, void*);
};

#endif
